from abc import ABC

from satquest.constants import CHARACTERS, CHEF_NAME, COOKIE_NAMES, GIT_HASH
from satquest.satquest_utils import get_class_source_hash


class Question(ABC):
    def visit_satdp(self, problem: "SATDP") -> str:
        # SAT Decision Problem
        pass

    def visit_satsp(self, problem: "SATSP") -> str:
        # SAT Solve Problem
        pass

    def visit_maxsat(self, problem: "MaxSAT") -> str:
        # MaxSAT
        pass

    def visit_mcs(self, problem: "MCS") -> str:
        # Minimal Correction Subset (MCS)
        pass

    def visit_mus(self, problem: "MUS") -> str:
        # Minimal Unsatisfiable Subset (MUS)
        pass

    def __repr__(self):
        return f"{self.__class__.__name__}_{GIT_HASH}_{get_class_source_hash(self.__class__)}"


class QuestionDIMACS(Question):
    def visit_satdp(self, problem: "SATDP") -> str:
        Q = self._get_q_prefix(problem)
        Q += f"""
Determine if the formula is satisfiable.
Output a binary string of length 1 ('1' for satisfiable, '0' for unsatisfiable)."""
        return Q

    def visit_satsp(self, problem: "SATSP") -> str:
        Q = self._get_q_prefix(problem)
        Q += f"""
Find a satisfying assignment for the formula.
Output a binary string of length {problem.cnf.nv} ('1' for true, '0' for false)."""
        return Q

    def visit_maxsat(self, problem: "MaxSAT") -> str:
        Q = self._get_q_prefix(problem)
        Q += f"""
Find an assignment that maximizes the number of satisfied clauses.
Output a binary string of length {problem.cnf.nv} ('1' for true, '0' for false)."""
        return Q

    def visit_mcs(self, problem: "MCS") -> str:
        Q = self._get_q_prefix(problem)
        Q += f"""
Find a minimal subset of clauses whose removal makes the formula satisfiable (no proper subset has this property).
Output a binary string of length {problem.cnf.mc} ('1' if the clause is in the subset, '0' otherwise), following the order of clauses in the formula."""
        return Q

    def visit_mus(self, problem: "MUS") -> str:
        Q = self._get_q_prefix(problem)
        Q += f"""
Find a minimal subset of clauses that is unsatisfiable (no proper subset is unsatisfiable).
Output a binary string of length {problem.cnf.mc} ('1' if the clause is in the subset, '0' otherwise), following the order of clauses in the formula."""
        return Q

    def _get_q_prefix(self, problem: "Problem") -> str:
        Q_prefix = f"""\
Given a CNF formula with {problem.cnf.nv} variables and {problem.cnf.mc} clauses in DIMACS format:

{problem.cnf.dimacs}
"""
        return Q_prefix


class QuestionMath(QuestionDIMACS):
    def _clauses2mathformula(self, clauses):
        return " \\land ".join(
            "("
            + " \\lor ".join(
                (f"x_{{{abs(lit)}}}" if lit > 0 else f"\\neg x_{{{abs(lit)}}}")
                if abs(lit) > 9
                else (f"x_{abs(lit)}" if lit > 0 else f"\\neg x_{abs(lit)}")
                for lit in clause
            )
            + ")"
            for clause in clauses
        )

    def _get_q_prefix(self, problem: "Problem") -> str:
        Q_prefix = f"""\
Given a CNF formula with {problem.cnf.nv} variables and {problem.cnf.mc} clauses in mathematical notation:

{self._clauses2mathformula(problem.cnf.clauses)}
"""
        return Q_prefix


class QuestionStory(Question):
    # Cookie Challenge (Wishes)

    def visit_satdp(self, problem: "SATDP") -> str:
        self._cookie_names, self._character_names = self._get_names(problem.cnf.nv, problem.cnf.mc)
        Q = self._get_q_prefix(problem)
        Q += f"""
Is it possible for Chef {CHEF_NAME} to bake cookies so every friend is happy?
Output a binary string of length 1 ('1' for yes, '0' for no)."""
        return Q

    def visit_satsp(self, problem: "SATSP") -> str:
        self._cookie_names, self._character_names = self._get_names(problem.cnf.nv, problem.cnf.mc)
        Q = self._get_q_prefix(problem)
        Q += f"""
Help Chef {CHEF_NAME} find a cookie recipe that makes everyone happy.
Output a binary string of length {problem.cnf.nv} for cookies ({', '.join(self._cookie_names)}): '1' for crunchy, '0' for chewy."""
        return Q

    def visit_maxsat(self, problem: "MaxSAT") -> str:
        self._cookie_names, self._character_names = self._get_names(problem.cnf.nv, problem.cnf.mc)
        Q = self._get_q_prefix(problem)
        Q += f"""
Help Chef {CHEF_NAME} find a cookie recipe that makes as many friends happy as possible.
Output a binary string of length {problem.cnf.nv} for cookies ({', '.join(self._cookie_names)}): '1' for crunchy, '0' for chewy."""
        return Q

    def visit_mcs(self, problem: "MCS") -> str:
        self._cookie_names, self._character_names = self._get_names(problem.cnf.nv, problem.cnf.mc)
        Q = self._get_q_prefix(problem)
        Q += f"""
Sadly, Chef {CHEF_NAME} can't make everyone happy. Find a minimal group of friends whose requirements Chef {CHEF_NAME} must ignore to keep the others happy. (This group is minimal: removing all requirements in this group is necessary to allow all other friends' requirements to be met, and removing only a part of this group is not sufficient.)
Output a binary string of length {problem.cnf.mc} ({', '.join(self._character_names)}): '1' to ignore their requirements, '0' otherwise."""
        return Q

    def visit_mus(self, problem: "MUS") -> str:
        self._cookie_names, self._character_names = self._get_names(problem.cnf.nv, problem.cnf.mc)
        Q = self._get_q_prefix(problem)
        Q += f"""
Sadly, Chef {CHEF_NAME} can't make everyone happy. Find a minimal group of friends such that Chef {CHEF_NAME} cannot possibly accommodate all their requirements at the same time. (This group is minimal: removing any single requirement from this group makes it possible to accommodate all the other requirements within this group.)
Output a binary string of length {problem.cnf.mc} ({', '.join(self._character_names)}): '1' if their requirements is part of this core group, '0' otherwise."""
        return Q

    def _get_q_prefix(self, problem: "Problem") -> str:
        Q_prefix = f"""It's cookie day on Quirkwild Zoo!
Chef {CHEF_NAME} is baking {problem.cnf.nv} kinds of cookies ({', '.join(self._cookie_names)}), each either crunchy or chewy.
Each of his {problem.cnf.mc} friends will be happy if {CHEF_NAME} bakes at least one cookie they prefer:

{self._clauses2story_conditions(problem.cnf.clauses)}
"""
        return Q_prefix

    def _clauses2story_conditions(self, clauses: list) -> str:
        friends_texts = []
        for friend_idx, clause in enumerate(clauses):
            friend_name = self._character_names[friend_idx]
            conditions = []
            for lit in clause:
                cookie_name = self._cookie_names[(abs(lit) - 1)]
                texture = "crunchy" if lit > 0 else "chewy"
                conditions.append(f"{texture} {cookie_name}")
            conditions_text = ", ".join(conditions)
            friends_texts.append(f"{friend_idx+1}. {friend_name} wants: {conditions_text}")
        friends_text = "\n".join(friends_texts)
        return friends_text

    def _get_names(self, n: int, m: int) -> tuple[list, list]:
        assert n <= len(COOKIE_NAMES)
        assert m <= len(CHARACTERS)
        return COOKIE_NAMES[:n], CHARACTERS[:m]


class QuestionDualStory(QuestionStory):
    # Cookie Challenge (Dislikes)

    def _get_q_prefix(self, problem: "Problem") -> str:
        Q_prefix = f"""It's cookie day on Quirkwild Zoo!
Chef {CHEF_NAME} is baking {problem.cnf.nv} kinds of cookies ({', '.join(self._cookie_names)}), each either crunchy or chewy.
Each of his {problem.cnf.mc} friends will be unhappy only if every cookie in their disliked combination is baked:

{self._clauses2story_conditions(problem.cnf.clauses)}
"""
        return Q_prefix

    def _clauses2story_conditions(self, clauses: list) -> str:
        friends_text = super()._clauses2story_conditions(clauses)
        friends_text = friends_text.replace("wants", "dislikes").replace(", ", " + ")
        # Reversed: positive -> chewy, negative -> crunchy
        friends_text = friends_text.replace("chewy", "==9527==").replace("crunchy", "chewy").replace("==9527==", "crunchy")
        return friends_text


def create_question(question_type: str) -> Question:
    question_type = question_type.lower()
    match question_type:
        case "dimacs":
            return QuestionDIMACS()
        case "math":
            return QuestionMath()
        case "story":
            return QuestionStory()
        case "dualstory":
            return QuestionDualStory()
    return None



import random

from pysat.formula import CNF as PysatCNF
from pysat.solvers import Solver

from satquest.constants import SAT_SOLVER_NAME


class CNF:
    def __init__(self, clauses: list = None, dimacs: str = None):
        assert clauses or dimacs
        if clauses:
            self.cnf = PysatCNF(from_clauses=clauses)
        else:
            self.cnf = PysatCNF(from_string=dimacs)
        self._is_sat = None

    @property
    def clauses(self) -> list:
        return self.cnf.clauses

    @property
    def nv(self) -> int:
        return self.cnf.nv

    @property
    def mc(self) -> int:
        return len(self.clauses)

    @property
    def dimacs(self) -> str:
        return self.cnf.to_dimacs()

    @property
    def is_sat(self) -> bool:
        if self._is_sat is None:
            with Solver(name=SAT_SOLVER_NAME, bootstrap_with=self.cnf) as solver:
                self._is_sat = solver.solve()
        return self._is_sat

    def shuffle(self, seed: int = None) -> None:
        self._is_sat = None
        _rng = random.Random(seed)
        for i in range(len(self.clauses)):
            _rng.shuffle(self.clauses[i])
        _rng.shuffle(self.clauses)

    def sort(self) -> None:
        self._is_sat = None
        for i in range(len(self.clauses)):
            self.clauses[i].sort(key=lambda x: abs(x))
        self.clauses.sort()



from abc import ABC, abstractmethod
from typing import Any, Generator

from pysat.examples.lbx import LBX as MCSSolver
from pysat.examples.musx import MUSX as MUSSolver
from pysat.examples.rc2 import RC2 as MaxSATSolver
from pysat.solvers import Solver

from satquest.cnf import CNF
from satquest.constants import GIT_HASH, SAT_SOLVER_NAME
from satquest.question import Question
from satquest.satquest_utils import cnf2wcnf, get_class_source_hash


class Problem(ABC):
    def __init__(self, cnf: CNF):
        self.cnf = cnf

    @property
    @abstractmethod
    def solution(self) -> Any:
        pass

    @abstractmethod
    def solution_enumerate(self) -> Generator[str, None, None]:
        pass

    @property
    @abstractmethod
    def search_space_size(self) -> Any:
        pass

    @abstractmethod
    def check(self, answer: Any) -> bool:
        pass

    @abstractmethod
    def format_check(self, answer: Any) -> bool:
        pass

    @abstractmethod
    def accept(self, question: Question, *args, **kwargs) -> str:
        pass

    @property
    @abstractmethod
    def ANSWER_PATTERN(self) -> str:
        pass

    @property
    def solver_metadata(self):
        if not hasattr(self, "_solver_metadata"):
            _ = self.solution
        return self._solver_metadata

    def __repr__(self):
        return f"{self.__class__.__name__}_{GIT_HASH}_{get_class_source_hash(self.__class__)}"


class SATDP(Problem):
    @property
    def solution(self) -> bool:
        if not hasattr(self, "_solution"):
            self._solution, self._solver_metadata = "-1", None
            try:
                with Solver(name=SAT_SOLVER_NAME, bootstrap_with=self.cnf.clauses) as solver:
                    self._solution = str(int(solver.solve()))
                    self._solver_metadata = {**solver.accum_stats(), "solvers": (SAT_SOLVER_NAME,)}
            except Exception:
                pass
        return self._solution

    def solution_enumerate(self) -> Generator[str, None, None]:
        yield self.solution

    @property
    def search_space_size(self) -> Any:
        return 2**self.cnf.nv

    def check(self, answer: str) -> bool:
        try:
            assert self.format_check(answer)
            return answer == str(self.solution)
        except Exception:
            pass
        return False

    def format_check(self, answer: str) -> bool:
        return isinstance(answer, str) and len(answer) == 1 and set(answer).issubset({"0", "1"})

    def accept(self, question: Question, *args, **kwargs) -> str:
        return question.visit_satdp(self, *args, **kwargs)

    @property
    def ANSWER_PATTERN(self) -> str:
        return r"(?=([01]{%d}))" % 1


class SATSP(Problem):
    @property
    def solution(self) -> str:
        assert self.cnf.is_sat
        if not hasattr(self, "_solution"):
            self._solution, self._solver_metadata = None, None
            try:
                with Solver(name=SAT_SOLVER_NAME, bootstrap_with=self.cnf.clauses) as solver:
                    _is_sat = solver.solve()
                    if _is_sat:
                        self._solution = "".join(["1" if iv > 0 else "0" for iv in solver.get_model()])
                    self._solver_metadata = {**solver.accum_stats(), "solvers": (SAT_SOLVER_NAME,)}
            except Exception:
                pass
        return self._solution

    def solution_enumerate(self) -> Generator[str, None, None]:
        assert self.cnf.is_sat
        with Solver(name=SAT_SOLVER_NAME, bootstrap_with=self.cnf.clauses) as solver:
            for s in solver.enum_models():
                yield "".join(["1" if iv > 0 else "0" for iv in s])

    @property
    def search_space_size(self) -> Any:
        return 2**self.cnf.nv

    def check(self, answer: str) -> bool:
        assert self.cnf.is_sat
        try:
            assert self.format_check(answer)
            with Solver(name=SAT_SOLVER_NAME, bootstrap_with=self.cnf.clauses) as solver:
                return solver.solve(assumptions=[(i + 1) if ai == "1" else -(i + 1) for i, ai in enumerate(answer)])
        except Exception:
            pass
        return False

    def format_check(self, answer: str) -> bool:
        return isinstance(answer, str) and len(answer) == self.cnf.nv and set(answer).issubset({"0", "1"})

    def accept(self, question: Question, *args, **kwargs) -> str:
        return question.visit_satsp(self, *args, **kwargs)

    @property
    def ANSWER_PATTERN(self) -> str:
        return r"(?=([01]{%d}))" % self.cnf.nv


class MaxSAT(Problem):
    @property
    def solution(self) -> str:
        assert not self.cnf.is_sat
        if not hasattr(self, "_solution"):
            self._solution, self._solver_metadata = None, None
            try:
                with MaxSATSolver(cnf2wcnf(self.cnf.cnf), solver=SAT_SOLVER_NAME, verbose=0) as solver:
                    solver.compute()
                    self._solver_metadata = {**solver.oracle.accum_stats(), "solvers": (SAT_SOLVER_NAME, "RC2")}
                    self._solution = "".join(["1" if iv > 0 else "0" for iv in solver.model])
            except Exception:
                pass
        return self._solution

    def solution_enumerate(self) -> Generator[str, None, None]:
        assert not self.cnf.is_sat
        from pysat.examples.rc2 import RC2

        with RC2(cnf2wcnf(self.cnf.cnf), solver=SAT_SOLVER_NAME, verbose=0) as solver:
            pre_cost = None
            for s in solver.enumerate():
                if pre_cost is None:
                    pre_cost = solver.cost
                if solver.cost != pre_cost:
                    break
                yield "".join(["1" if iv > 0 else "0" for iv in s])

    @property
    def search_space_size(self) -> Any:
        return 2**self.cnf.nv

    def check(self, answer: str) -> bool:
        assert not self.cnf.is_sat
        try:
            assert self.format_check(answer)
            from pysat.examples.lsu import LSU as MaxSATSolver

            with MaxSATSolver(cnf2wcnf(self.cnf.cnf), solver=SAT_SOLVER_NAME, verbose=0) as solver:
                solver.solve()
                answer_cost = solver._get_model_cost(
                    cnf2wcnf(self.cnf.cnf), [(i + 1) if ai == "1" else -(i + 1) for i, ai in enumerate(answer)]
                )
                return answer_cost == solver.cost
        except Exception:
            pass
        return False

    def format_check(self, answer: str) -> bool:
        return isinstance(answer, str) and len(answer) == self.cnf.nv and set(answer).issubset({"0", "1"})

    def accept(self, question: Question, *args, **kwargs) -> str:
        return question.visit_maxsat(self, *args, **kwargs)

    @property
    def ANSWER_PATTERN(self) -> str:
        return r"(?=([01]{%d}))" % self.cnf.nv


class MCS(Problem):
    @property
    def solution(self) -> str:
        assert not self.cnf.is_sat
        if not hasattr(self, "_solution"):
            with MCSSolver(cnf2wcnf(self.cnf.cnf), use_cld=False, solver_name=SAT_SOLVER_NAME) as solver:
                _solution_model = solver.compute()
                self._solution = "".join(["1" if i in _solution_model else "0" for i in range(1, self.cnf.mc + 1)])
                self._solver_metadata = {**solver.oracle.accum_stats(), "solvers": (SAT_SOLVER_NAME, "LBX")}
        return self._solution

    def solution_enumerate(self) -> Generator[str, None, None]:
        assert not self.cnf.is_sat
        with MCSSolver(cnf2wcnf(self.cnf.cnf), use_cld=False, solver_name=SAT_SOLVER_NAME) as solver:
            for mcs in solver.enumerate():
                solver.block(mcs)
                yield "".join(["1" if i in mcs else "0" for i in range(1, len(self.cnf.clauses) + 1)])

    @property
    def search_space_size(self) -> Any:
        return 2**self.cnf.mc

    def check(self, answer: str) -> bool:
        assert not self.cnf.is_sat
        try:
            assert self.format_check(answer)
            unmcs_clauses = [self.cnf.clauses[i] for i in range(self.cnf.mc) if answer[i] == "0"]
            mcs_clauses = [self.cnf.clauses[i] for i in range(self.cnf.mc) if answer[i] == "1"]
            with Solver(name=SAT_SOLVER_NAME, bootstrap_with=CNF(unmcs_clauses).cnf) as solver:
                if not solver.solve():
                    return False
            for i in range(len(mcs_clauses)):
                add_mcs_clauses = unmcs_clauses + [mcs_clauses[i]]
                with Solver(name=SAT_SOLVER_NAME, bootstrap_with=CNF(add_mcs_clauses).cnf) as solver:
                    if solver.solve():
                        return False
            return True
        except Exception:
            pass
        return False

    def format_check(self, answer: str) -> bool:
        return isinstance(answer, str) and len(answer) == self.cnf.mc and set(answer).issubset({"0", "1"})

    def accept(self, question: Question, *args, **kwargs) -> str:
        return question.visit_mcs(self, *args, **kwargs)

    @property
    def ANSWER_PATTERN(self) -> str:
        return r"(?=([01]{%d}))" % self.cnf.mc


class MUS(Problem):
    @property
    def solution(self) -> str:
        assert not self.cnf.is_sat
        if not hasattr(self, "_solution"):
            with MUSSolver(self.cnf.cnf, solver=SAT_SOLVER_NAME, verbosity=0) as solver:
                _solution_model = solver.compute()
                self._solution = "".join(["1" if i in _solution_model else "0" for i in range(1, self.cnf.mc + 1)])
                self._solver_metadata = {**solver.oracle.accum_stats(), "solvers": (SAT_SOLVER_NAME, "MUSX")}
        return self._solution

    def solution_enumerate(self) -> Generator[str, None, None]:
        assert not self.cnf.is_sat
        from pysat.examples.hitman import Hitman

        with Hitman(solver="m22") as hitman_solver:
            with MCSSolver(cnf2wcnf(self.cnf.cnf), use_cld=False, solver_name=SAT_SOLVER_NAME) as solver:
                for mcs in solver.enumerate():
                    solver.block(mcs)
                    hitman_solver.hit(mcs)
            for mus in hitman_solver.enumerate():
                yield "".join(["1" if i in mus else "0" for i in range(1, len(self.cnf.clauses) + 1)])

    @property
    def search_space_size(self) -> Any:
        return 2**self.cnf.mc

    def check(self, answer: str) -> bool:
        assert not self.cnf.is_sat
        try:
            assert self.format_check(answer)
            mus_clauses = [self.cnf.clauses[i] for i in range(self.cnf.mc) if answer[i] == "1"]
            with Solver(name=SAT_SOLVER_NAME, bootstrap_with=CNF(mus_clauses).cnf) as solver:
                if solver.solve():
                    return False
            for i in range(len(mus_clauses)):
                sub_mus_clauses = mus_clauses[:i] + mus_clauses[i + 1 :]
                with Solver(name=SAT_SOLVER_NAME, bootstrap_with=CNF(sub_mus_clauses).cnf) as solver:
                    if not solver.solve():
                        return False
            return True
        except Exception:
            pass
        return False

    def format_check(self, answer: str) -> bool:
        return isinstance(answer, str) and len(answer) == self.cnf.mc and set(answer).issubset({"0", "1"})

    def accept(self, question: Question, *args, **kwargs) -> str:
        return question.visit_mus(self, *args, **kwargs)

    @property
    def ANSWER_PATTERN(self) -> str:
        return r"(?=([01]{%d}))" % self.cnf.mc


def create_problem(problem_type: str, cnf: CNF) -> Problem:
    problem_type = problem_type.lower()
    match problem_type:
        case "satdp" | "satdp_sat" | "satdp_unsat":
            return SATDP(cnf)
        case "satsp":
            return SATSP(cnf)
        case "maxsat":
            return MaxSAT(cnf)
        case "mcs":
            return MCS(cnf)
        case "mus":
            return MUS(cnf)
    return None



#!/usr/bin/env python
# setup.py
from setuptools import find_packages, setup

setup(
    name="satquest",
    version="0.1.1",
    author="Yanxiao Zhao",
    author_email="pazyx728@gmail.com",
    description="SATQuest",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/sdpkjc/SATQuest",
    packages=find_packages("."),
    python_requires=">=3.9",
    install_requires=[
        "python-sat==1.8.dev14",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
)



import hashlib
import inspect
import re

from pysat.formula import WCNF


def cnf2wcnf(cnf):
    wcnf = WCNF()
    wcnf.nv = cnf.nv
    for clause in cnf.clauses:
        wcnf.append(clause, weight=1)
    return wcnf


def get_class_source_hash(cls):
    try:
        source = inspect.getsource(cls)
        source_hash = hashlib.md5(source.encode("utf-8")).hexdigest()[:8]
    except Exception:
        source_hash = "unknown"

    return source_hash


def re_matcher(content_output: str, pattern: str) -> str:
    final_answer = None
    try:
        match = re.finditer(pattern, content_output, re.DOTALL)
        matches = list(match)
        if matches:
            final_answer = matches[-1].group(1)
    except Exception:
        pass
    return final_answer


SYSTEM_PROMPT = "You are a helpful assistant."
QUERY_TEMPLATE = """\
Solve the following problem step by step. The last line of your response should be of the form Answer: $ANSWER (without quotes) where $ANSWER is the answer to the problem.

{Question}

Remember to put your answer on its own line after "Answer:", and you do not need to use a \\boxed command.
""".strip()

ANSWER_PATTERN = r"(?i)answer:\s*([^\n]+)"



import os
import subprocess

try:
    GIT_HASH = subprocess.check_output(["git", "rev-parse", "--short", "HEAD"]).strip().decode("utf-8")
except Exception:
    GIT_HASH = "unknown"

G_FILE_PATH = os.path.join(os.path.dirname(__file__), "..", "graphs_3_7_c.g6")

SAT_SOLVER_NAME = "g4"

COOKIE_NAMES = [
    "oatmeal",
    "chocolate chip",
    "peanut butter",
    "sugar",
    "gingerbread",
    "shortbread",
    "snickerdoodle",
    "macadamia",
    "raisin",
    "butter",
    "almond",
    "hazelnut",
    "cinnamon",
    "lemon",
    "coconut",
    "coffee",
    "caramel",
    "pumpkin",
    "mint",
    "vanilla",
    "pistachio",
    "nutmeg",
    "pecan",
    "marzipan",
    "banana",
    "cranberry",
    "blueberry",
    "lime",
    "apricot",
    "walnut",
    "fig",
    "maple",
    "orange",
    "lavender",
    "cherry",
    "chai",
    "mocha",
    "sesame",
    "molasses",
    "matcha",
    "apple",
    "blackberry",
    "cardamom",
    "date",
    "espresso",
    "anise",
    "papaya",
    "praline",
    "pear",
    "rose",
    "toffee",
    "ube",
    "kiwi",
    "guava",
    "plum",
    "tapioca",
    "yam",
    "mango",
    "oreo",
    "figgy",
    "brandy",
    "rum",
    "thyme",
    "spice",
]


CHARACTERS = [
    "Quoll",
    "Skua",
    "Falcon",
    "Jacana",
    "Whimbrel",
    "Anole",
    "Coelacanth",
    "Bowerbird",
    "Mantis",
    "Neonate",
    "Heron",
    "Barnacle",
    "Puffbird",
    "Platypus",
    "Redstart",
    "Mola",
    "Penguin",
    "Cockatoo",
    "Fennec",
    "Anglerfish",
    "Hyrax",
    "Gecko",
    "Howler",
    "Katydid",
    "Monarch",
    "Cardinal",
    "Pangolin",
    "Irrawaddy",
    "Anemone",
    "Gibbon",
    "Magpie",
    "Pipefish",
    "Ayeaye",
    "Mandarin",
    "Jerboa",
    "Hoopoe",
    "Kingfisher",
    "Chough",
    "Lemming",
    "Batis",
    "Drill",
    "Bushbaby",
    "Nuthatch",
    "Quokka",
    "Iora",
    "Swiftlet",
    "Gerenuk",
    "Mink",
    "Ixodes",
    "Bobolink",
    "Dog",
    "Chipmunk",
    "Orca",
    "Galah",
    "Platybe",
    "Numbat",
    "Grackle",
    "Manakin",
    "Dory",
    "Quail",
    "Dikdik",
    "Kinkajou",
    "Squid",
    "Lacewing",
    "Tenrec",
    "Noddle",
    "Frigatebird",
    "Nautilus",
    "Olingo",
    "Baboon",
    "Pelican",
    "Cardinalfish",
    "Oystercatcher",
    "Finch",
    "Caddisfly",
    "Flycatcher",
    "Jacamar",
    "Aphid",
    "Crossbill",
    "Drongo",
    "Nehushtan",
    "Saki",
    "Lapwing",
    "Koala",
    "Ibis",
    "Nerodia",
    "Kangaroo",
    "Caecilian",
    "Lemur",
    "Butterfly",
    "Marten",
    "Longspur",
    "Bobcat",
    "Krill",
    "Peregrine",
    "Blenny",
    "Tapir",
    "Bilby",
    "Fairyshrimp",
    "Puffin",
    "Goeldi",
    "Bonobo",
    "Cuttlefish",
    "Narwhal",
    "Quicon",
    "Isopod",
    "Harrier",
    "Chinchilla",
    "Eland",
    "Tortoise",
    "Fox",
    "Flamingo",
    "Macaw",
    "Potoroo",
    "Starfish",
    "Stonefly",
    "Icterine",
    "Condor",
    "Markhor",
    "Inia",
    "Dobsonfly",
    "Bulbul",
    "Avocet",
    "Junco",
    "Sifaka",
    "Dovekie",
    "Ixia",
    "Barbet",
    "Kudu",
    "Oriole",
    "Budgie",
    "Damselfly",
    "Quetzal",
    "Antbird",
    "Ladybird",
    "Lyrebird",
    "Myna",
    "Quelea",
    "Meerkat",
    "Urchin",
    "Springhare",
    "Plover",
    "Fieldfare",
    "Qualis",
    "Moorhen",
    "Fluke",
    "Hoatzin",
    "Iulu",
    "Canary",
    "Lark",
    "Colobus",
    "Pika",
    "Kestrel",
    "Capybara",
    "Coati",
    "Dormouse",
    "Anteater",
    "Pipistrelle",
    "Quadrula",
    "Dunnart",
    "Leafy",
    "Firefly",
    "Quimera",
    "Manx",
    "Kiwi",
    "Osprey",
    "Dhole",
    "Ichneumon",
    "Panda",
    "Egret",
    "Jaguarundi",
    "SugarGlider",
    "Angelfish",
    "Sunbear",
    "Gopher",
    "Goby",
    "Jabiru",
    "Kinglet",
    "Wallaroo",
    "Owl",
    "Skylark",
    "Echidna",
    "Otter",
    "Dingo",
    "Peacock",
    "Terrapin",
    "Erythrocharax",
    "Beluga",
    "Nicator",
    "Loris",
    "Quahog",
    "Redpoll",
    "Grosbeak",
    "Capuchin",
    "Hamster",
    "Jay",
    "Tamarin",
    "Junonia",
    "Butterflyfish",
    "Jellyfish",
    "Bettong",
    "Bunting",
    "Nightjar",
    "Eagle",
    "Neurobasis",
    "Gannet",
    "Tigon",
    "Wagtail",
    "Jackdaw",
    "Peccary",
    "Stormpetrel",
    "Pipit",
    "Manul",
    "Hornbill",
    "Octopus",
    "Ferret",
    "Porpoise",
    "Albatross",
    "Chameleon",
    "Spatangoida",
    "Marmot",
    "Blobfish",
    "Fantail",
    "Erinaceus",
    "Weaver",
    "Caracal",
    "Roadrunner",
    "Titi",
    "Goldfish",
    "Mockingbird",
    "Seadragon",
    "Moth",
    "Hawk",
    "Dragonfly",
    "Ocelot",
    "Uakari",
    "Cuckoo",
    "Aardvark",
    "Vaquita",
    "Alpaca",
    "Nudibranch",
    "Weta",
    "Bumblebee",
    "Fairywren",
    "Emperor",
    "Bittern",
    "Puffleg",
    "Dotterel",
    "Caracara",
    "Pangasius",
    "Euploea",
    "Wolverine",
    "Buzzard",
    "Nightingale",
    "Oryx",
    "Earwig",
    "Toucan",
    "Motmot",
    "Discus",
    "Parrot",
    "Axolotl",
    "Kittiwake",
    "Leipoa",
    "Ibycter",
    "Gerbil",
    "Javan",
    "Mandrill",
    "Dunlin",
    "Hedgehog",
    "Lorikeet",
    "Gourami",
    "Okapi",
    "Clownfish",
    "Kookaburra",
    "Eryops",
    "Euphlyctis",
    "Iguana",
    "Hummingbird",
    "Eubalaena",
    "Cicada",
    "Quagga",
    "Guppy",
    "Clingfish",
    "Marmoset",
    "Bandicoot",
]
CHEF_NAME, CHARACTERS = CHARACTERS[0], CHARACTERS[1:]
